@@ -8,7 +8,7 @@ from seqmagick import __version__ as version
 from seqmagick import subcommands
 
 
-def main(argv):
+def main(argv=sys.argv[1:]):
     action, arguments = parse_arguments(argv)
 
     loglevel = {
@@ -56,7 +56,8 @@ def parse_arguments(argv):
     # Add actions
     actions = {}
     for name, mod in subcommands.itermodules():
-        subparser = subparsers.add_parser(name, help=mod.__doc__)
+        subparser = subparsers.add_parser(name, help=mod.__doc__,
+                description=mod.__doc__)
         mod.build_parser(subparser)
         actions[name] = mod.action
 
