@@ -2,16 +2,25 @@
 Convert between sequence formats
 """
 import argparse
+import functools
 import logging
 import os.path
 
-from Bio import SeqIO
+from Bio import Alphabet, SeqIO
+from Bio.Alphabet import IUPAC
 from Bio.SeqIO import FastaIO
 from seqmagick import transform
 from seqmagick.fileformat import from_extension
 
-import common
+from . import common
 
+ALPHABETS = {
+        'dna': Alphabet.generic_dna,
+        'dna-ambiguous': IUPAC.ambiguous_dna,
+        'protein': Alphabet.generic_protein,
+        'rna': Alphabet.generic_rna,
+        'rna-ambiguous': IUPAC.ambiguous_rna,
+}
 
 def add_options(parser):
     """
@@ -30,36 +39,54 @@ def add_options(parser):
 
     seq_mods = parser.add_argument_group("Sequence Modificaton")
     seq_mods.add_argument('--apply-function', type=module_function,
-            metavar='/path/to/module.py:function_name',
+            metavar='/path/to/module.py:function_name[:parameter]',
             help="""Specify a custom function to apply to the input sequences,
             specified as /path/to/file.py:function_name. Function should accept
-            an iterable of Bio.SeqRecord objects, and yield SeqRecords.
-            Specify more than one to chain.""",
+            an iterable of Bio.SeqRecord objects, and yield SeqRecords. If the
+            parameter is specified, it will be passed as a string as the second
+            argument to the function. Specify more than one to chain.""",
             default=[], action='append')
-    seq_mods.add_argument('--cut', dest='transforms', metavar="start:end",
-            type=common.cut_range,
-            action=partial_action(transform.cut_sequences, 'cut_slice'),
-            help="""1-indexed start and end positions for cutting sequences, :
-            separated. Includes last item.""")
+    seq_mods.add_argument('--cut', dest='transforms',
+            metavar="start:end[,start2:end2]",
+            type=common.sequence_slices,
+            action=partial_action(transform.multi_cut_sequences, 'slices'),
+            help="""Keep only the residues within the 1-indexed start and end
+            positions specified, : separated. Includes last item. Start or end
+            can be left unspecified to indicate start/end of sequence.""")
+    seq_mods.add_argument('--relative-to', dest='cut_relative', metavar='ID',
+            help="""Apply --cut relative to the indexes of non-gap residues in
+            sequence identified by ID""")
+    seq_mods.add_argument('--drop', dest='transforms',
+            metavar='start:end[,start2:end2]',
+            type=common.sequence_slices,
+            action=partial_action(transform.drop_columns, 'slices'),
+            help="""Remove the residues at the specified indices. Same format as `--cut`.""")
     seq_mods.add_argument('--dash-gap',
             action=partial_action(transform.dashes_cleanup), dest='transforms',
-        help='Change . and : into - for all sequences')
+            help="""Replace any of the characters "?.:~" with a "-" for all
+            sequences""")
     seq_mods.add_argument('--lower',
             action=partial_action(transform.lower_sequences),
             dest='transforms', help='Translate the sequences to lower case')
+    seq_mods.add_argument('--mask', metavar="start1:end1[,start2:end2]",
+            action=partial_action(transform.multi_mask_sequences, 'slices'),
+            type=common.sequence_slices, dest='transforms', help="""Replace
+            residues in 1-indexed slice with gap-characters. If --relative-to
+            is also specified, coordinates are relative to the sequence ID
+            provided.""")
     seq_mods.add_argument('--reverse',
             action=partial_action(transform.reverse_sequences),
             dest='transforms', help='Reverse the order of sites in sequences')
     seq_mods.add_argument('--reverse-complement', dest='transforms',
             action=partial_action(transform.reverse_complement_sequences),
             help='Convert sequences into reverse complements')
-    seq_mods.add_argument('--squeeze', action='store_const', dest='squeeze',
-            default=None,
-            const=1.0, help='''Remove any gaps that are present in the same
+    seq_mods.add_argument('--squeeze', action=partial_action(transform.squeeze),
+            dest='transforms',
+            help='''Remove any gaps that are present in the same
             position across all sequences in an alignment (equivalent to
             --squeeze-threshold=1.0)''')
-    seq_mods.add_argument('--squeeze-threshold', dest='squeeze',
-            action="store",
+    seq_mods.add_argument('--squeeze-threshold', dest='transforms',
+            action=partial_action(transform.squeeze, 'gap_threshold'),
             type=common.typed_range(float, 0.0, 1.0),
             metavar='PROP', help="""Trim columns from an alignment which
             have gaps in least the specified proportion of sequences.""")
@@ -142,7 +169,8 @@ def add_options(parser):
             action=partial_action(transform.seq_exclude, 'filter_regex'),
             dest='transforms', help="""Filter the sequences by regular
             expression in sequence""")
-    seq_select.add_argument('--tail', metavar='N', dest='tail', type=int,
+    seq_select.add_argument('--tail', metavar='N', dest='transforms', type=int,
+            action=partial_action(transform.tail, 'tail'),
         help='Trim down to bottom N sequences')
 
     id_mods = parser.add_argument_group("Sequence ID Modification")
@@ -173,6 +201,9 @@ def add_options(parser):
     format_group.add_argument('--output-format', metavar='Format',
             help="Output file format (default: determine from extension)")
 
+    parser.add_argument('--alphabet', choices=ALPHABETS,
+            help="""Input alphabet. Required for writing NEXUS.""")
+
     return parser
 
 
@@ -183,8 +214,7 @@ def build_parser(parser):
     add_options(parser)
     parser.add_argument('source_file', type=argparse.FileType('r'),
                         help="Input sequence file")
-    parser.add_argument('dest_file', type=argparse.FileType('w'),
-                        help="Output file")
+    parser.add_argument('dest_file', help="Output file")
 
     return parser
 
@@ -193,7 +223,8 @@ def transform_file(source_file, destination_file, arguments):
     file_ext = os.path.splitext(source_file.name)[1]
     source_file_type = (arguments.input_format or from_extension(file_ext))
 
-    output_ext = os.path.splitext(destination_file.name)[1]
+    output_ext = os.path.splitext(getattr(destination_file, 'name', ''))[1]
+
     destination_file_type = (arguments.output_format or
             from_extension(output_ext))
 
@@ -209,17 +240,36 @@ def transform_file(source_file, destination_file, arguments):
                 direction=directions[direction])
     else:
         # Unsorted iterator.
-        records = SeqIO.parse(source_file, source_file_type)
+        records = SeqIO.parse(source_file, source_file_type,
+                alphabet=ALPHABETS.get(arguments.alphabet))
 
 
     #########################################
     # Apply generator functions to iterator.#
     #########################################
 
-    logging.info('Setting up transform functions for file: %s', source_file)
-
     # Apply all the transform functions in transforms
     if arguments.transforms:
+
+        # Special case handling for --cut and --relative-to
+        if arguments.cut_relative:
+            for o, n in ((transform.multi_cut_sequences,
+                          transform.cut_sequences_relative),
+                         (transform.multi_mask_sequences,
+                          transform.mask_sequences_relative)):
+                # Add a function to trim any columns which are gaps in the
+                # sequence ID
+                try:
+                    f = next(f for f in arguments.transforms
+                             if f.func == o)
+                except StopIteration:
+                    continue
+                i = arguments.transforms.index(f)
+                arguments.transforms.pop(i)
+                arguments.transforms.insert(i,
+                        functools.partial(n,
+                            record_id=arguments.cut_relative, **f.keywords))
+
         for function in arguments.transforms:
             records = function(records)
 
@@ -228,23 +278,11 @@ def transform_file(source_file, destination_file, arguments):
         records = transform.deduplicate_sequences(
             records, arguments.deduplicate_sequences)
 
+    # Apply all the partial functions
     if arguments.apply_function:
         for apply_function in arguments.apply_function:
             records = apply_function(records)
 
-
-    if arguments.tail:
-        # To know where to begin including records for tail, we need to count
-        # the total number of records, which requires going through the entire
-        # file and additional time.
-        record_count = sum(1 for record in SeqIO.parse(source_file, source_file_type))
-        records = transform.tail(records, arguments.tail, record_count)
-
-    if arguments.squeeze:
-        logging.info("Performing squeeze")
-        records = transform.squeeze(records, arguments.squeeze,
-                                    SeqIO.parse(source_file, source_file_type))
-
     # Only the fasta format is supported, as SeqIO.write does not have a 'wrap'
     # parameter.
     if (arguments.line_wrap is not None and destination_file_type == 'fasta'
@@ -272,12 +310,13 @@ def module_function(string):
     """
     Load a function from a python module using a file name, function name
     specification of format:
-        /path/to/x.py:function_name
+        /path/to/x.py:function_name[:parameter]
     """
-    parts = string.split(':')
-    if len(parts) != 2:
-        raise ValueError("Illegal specification. Should be module:function")
-    module_path, function_name = parts
+    parts = string.split(':', 2)
+    if len(parts) < 2:
+        raise ValueError(
+            "Illegal specification. Should be module:function[:parameter]")
+    module_path, function_name = parts[:2]
 
     # Import the module
     module_vars = {}
@@ -289,13 +328,14 @@ def module_function(string):
         raise argparse.ArgumentTypeError("{0} has no attribute '{1}'".format(
             module_path, function_name))
 
-    # Must be callable
-    if not callable(function):
-        raise argparse.ArgumentTypeError("{0} is not callable.".format(
-            function_name))
+    if len(parts) == 3:
+        old_function = function
+        function = lambda r: old_function(r, parts[2])
 
     return function
 
 
 def action(arguments):
-    transform_file(arguments.source_file, arguments.dest_file, arguments)
+    with arguments.source_file as src, \
+            common.atomic_write(arguments.dest_file) as dest:
+        transform_file(src, dest, arguments)
