@@ -2,25 +2,59 @@
 Functions to transform / filter sequences
 """
 import collections
+import contextlib
+import cPickle as pickle
 import itertools
 import logging
 import re
 import string
+import tempfile
 
 from Bio import SeqIO
-from Bio.Alphabet import generic_dna, generic_rna
+from Bio.Alphabet import IUPAC
+from Bio.Data import CodonTable
 from Bio.Seq import Seq
 from Bio.SeqRecord import SeqRecord
 from Bio.SeqUtils.CheckSum import seguid
 
+# Characters to be treated as gaps
+GAP_CHARS = "-."
 
-def dashes_cleanup(records):
+# Size of temporary file buffer: default to 20MB
+DEFAULT_BUFFER_SIZE = 20971520 # 20*2**20
+
+@contextlib.contextmanager
+def _record_buffer(records, buffer_size=DEFAULT_BUFFER_SIZE):
+    """
+    Buffer for transform functions which require multiple passes through data.
+
+    Value returned by context manager is a function which returns an iterator
+    through records.
+    """
+    with tempfile.SpooledTemporaryFile(buffer_size, mode='wb+') as tf:
+        pickler = pickle.Pickler(tf)
+        for record in records:
+            pickler.dump(record)
+
+        def record_iter():
+            tf.seek(0)
+            unpickler = pickle.Unpickler(tf)
+            while True:
+                try:
+                    yield unpickler.load()
+                except EOFError:
+                    break
+
+        yield record_iter
+
+
+def dashes_cleanup(records, prune_chars='.:?~'):
     """
     Take an alignment and convert any undesirable characters such as ? or ~ to
     -.
     """
-    logging.info("Applying _dashes_cleanup: converting any ? or ~ to -.")
-    translation_table = string.maketrans("?~", "--")
+    logging.info("Applying _dashes_cleanup: converting any . or : to -.")
+    translation_table = string.maketrans(prune_chars, '-' * len(prune_chars))
     for record in records:
         record.seq = Seq(str(record.seq).translate(translation_table),
                          record.seq.alphabet)
@@ -134,13 +168,109 @@ def isolate_region(sequences, start, end, gap_char='-'):
         yield sequence
 
 
-def cut_sequences(records, cut_slice):
+def _cut_sequences(records, cut_slice):
     """
     Cut sequences given a slice.
     """
     for record in records:
         yield record[cut_slice]
 
+def drop_columns(records, slices):
+    """
+    Drop all columns present in ``slices`` from records
+    """
+    for record in records:
+        # Generate a set of indices to remove
+        drop = set(i for slice in slices
+                   for i in range(*slice.indices(len(record))))
+        keep = [i not in drop for i in xrange(len(record))]
+        record.seq = Seq(''.join(itertools.compress(record.seq, keep)), record.seq.alphabet)
+        yield record
+
+def multi_cut_sequences(records, slices):
+    # If only a single slice is specified, use _cut_sequences,
+    # since this preserves per-letter annotations
+    if len(slices) == 1:
+        for sequence in _cut_sequences(records, slices[0]):
+            yield sequence
+    else:
+        # For multiple slices, concatenate the slice results
+        for record in records:
+            pieces = (record[s] for s in slices)
+            # SeqRecords support addition as concatenation
+            yield reduce(lambda x, y: x + y, pieces)
+
+def _update_slices(record, slices):
+    n = itertools.count().next
+    # Generate a map from indexes in the specified sequence to those in the
+    # alignment
+    ungap_map = dict((n(), i) for i, base in enumerate(str(record.seq))
+                     if base not in GAP_CHARS)
+    def update_slice(s):
+        """
+        Maps a slice relative to ungapped record_id to a slice valid for the
+        whole alignment.
+        """
+        start, end = s.start, s.stop
+        if start is not None:
+            try:
+                start = ungap_map[start]
+            except KeyError:
+                raise KeyError("""No index {0} in {1}.""".format(
+                    start, record.id))
+        if end is not None:
+            # We need the base in the slice identified by end, not the base
+            # at end, otherwise insertions between end-1 and end will be
+            # included.
+            try:
+                end = ungap_map[end - 1] + 1
+            except KeyError:
+                logging.warn("""No index %d in %s. Keeping columns to end
+                    of alignment.""", end, record.id)
+                end = None
+
+        return slice(start, end)
+
+    return [update_slice(s) for s in slices]
+
+def cut_sequences_relative(records, slices, record_id):
+    """
+    Cuts records to slices, indexed by non-gap positions in record_id
+    """
+    with _record_buffer(records) as r:
+        try:
+            record = next(i for i in r() if i.id == record_id)
+        except StopIteration:
+            raise ValueError("Record with id {0} not found.".format(record_id))
+
+        new_slices = _update_slices(record, slices)
+        for record in multi_cut_sequences(r(), new_slices):
+            yield record
+
+def multi_mask_sequences(records, slices):
+    """
+    Replace characters sliced by slices with gap characters.
+    """
+    for record in records:
+        record_indices = range(len(record))
+        keep_indices = reduce(lambda i, s: i - frozenset(record_indices[s]),
+                              slices, frozenset(record_indices))
+        seq = ''.join(b if i in keep_indices else '-'
+                      for i, b in enumerate(str(record.seq)))
+        record.seq = Seq(seq)
+        yield record
+
+def mask_sequences_relative(records, slices, record_id):
+    with _record_buffer(records) as r:
+        try:
+            record = next(i for i in r() if i.id == record_id)
+        except StopIteration:
+            raise ValueError("Record with id {0} not found.".format(record_id))
+
+        new_slices = _update_slices(record, slices)
+        for record in multi_mask_sequences(r(), new_slices):
+            yield record
+
 
 def lower_sequences(records):
     """
@@ -223,15 +353,31 @@ def reverse_complement_sequences(records):
         yield rev_record
 
 
-def ungap_sequences(records):
+def ungap_sequences(records, gap_chars=GAP_CHARS):
     """
     Remove gaps from sequences, given an alignment.
     """
     logging.info('Applying _ungap_sequences generator: '
                  'removing gaps from the alignment.')
     for record in records:
-        yield SeqRecord(record.seq.ungap("-"), id=record.id,
-                        description=record.description)
+        yield ungap_all(record, gap_chars)
+
+def ungap_all(record, gap_chars=GAP_CHARS):
+    record = SeqRecord(Seq(str(record.seq).translate(None, gap_chars)),
+            id=record.id, description=record.description)
+    return record
+
+def _update_id(record, new_id):
+    """
+    Update a record id to new_id, also modifying the ID in record.description
+    """
+    old_id = record.id
+    record.id = new_id
+
+    # At least for FASTA, record ID starts the description
+    record.description = re.sub('^' + re.escape(old_id), new_id,
+            record.description)
+    return record
 
 
 def name_append_suffix(records, suffix):
@@ -242,8 +388,9 @@ def name_append_suffix(records, suffix):
                  'Appending suffix ' + suffix + ' to all '
                  'sequence IDs.')
     for record in records:
-        yield SeqRecord(record.seq, id=record.id+suffix,
-                        description=record.description)
+        new_id = record.id + suffix
+        _update_id(record, new_id)
+        yield record
 
 
 def name_insert_prefix(records, prefix):
@@ -254,8 +401,9 @@ def name_insert_prefix(records, prefix):
                  'Inserting prefix ' + prefix + ' for all '
                  'sequence IDs.')
     for record in records:
-        yield SeqRecord(record.seq, id=prefix+record.id,
-                        description=record.description)
+        new_id = prefix + record.id
+        _update_id(record, new_id)
+        yield record
 
 
 
@@ -269,7 +417,7 @@ def name_include(records, filter_regex):
                  ' in results.')
     regex = re.compile(filter_regex, re.I)
     for record in records:
-        if regex.search(record.id):
+        if regex.search(record.id) or regex.search(record.description):
             yield record
 
 
@@ -282,7 +430,7 @@ def name_exclude(records, filter_regex):
                  'excluding IDs matching ' + filter_regex + ' in results.')
     regex = re.compile(filter_regex, re.I)
     for record in records:
-        if not regex.search(record.id):
+        if not regex.search(record.id) and not regex.search(record.description):
             yield record
 
 
@@ -327,22 +475,17 @@ def head(records, head):
     return itertools.islice(records, head)
 
 
-def tail(records, tail, record_count):
+def tail(records, tail):
     """
     Limit results to the bottom N records.
     """
-    logging.info('Applying _tail generator: '
-                 'limiting results to bottom ' + str(tail) + ' records.')
-    position = 0
-    start = record_count - tail
-    for record in records:
-        if position < start:
-            position += 1
-            continue
-        else:
+    with _record_buffer(records) as r:
+        record_count = sum(1 for record in r())
+        start_index = record_count - tail
+        rec_iter = r()
+        for record in itertools.islice(rec_iter, start_index, None):
             yield record
 
-
 # Squeeze-related
 def gap_proportion(sequences, gap_chars='-'):
     """
@@ -370,26 +513,23 @@ def gap_proportion(sequences, gap_chars='-'):
     return gap_props
 
 
-def squeeze(records, gap_threshold, sequence_iterator):
+def squeeze(records, gap_threshold=1.0):
     """
     Remove any gaps that are present in the same position across all sequences
     in an alignment.  Takes a second sequence iterator for determining gap
     positions.
     """
-    logging.info('Applying squeeze: '
-                 'removing any gaps that are present '
-                 'in the same position across all sequences in an alignment.')
+    with _record_buffer(records) as r:
+        gap_proportions = gap_proportion(r())
 
-    gap_proportions = gap_proportion(sequence_iterator)
-    keep_columns = [g < gap_threshold for g in gap_proportions]
-
-    for record in records:
-        sequence = str(record.seq)
-        # Trim
-        squeezed = itertools.compress(sequence, keep_columns)
-        yield SeqRecord(Seq(''.join(squeezed)), id=record.id,
-                        description=record.description)
+        keep_columns = [g < gap_threshold for g in gap_proportions]
 
+        for record in r():
+            sequence = str(record.seq)
+            # Trim
+            squeezed = itertools.compress(sequence, keep_columns)
+            yield SeqRecord(Seq(''.join(squeezed)), id=record.id,
+                            description=record.description)
 
 def strip_range(records):
     """
@@ -427,14 +567,42 @@ def transcribe(records, transcribe):
         description = record.description
         name = record.id
         if transcribe == 'dna2rna':
-            dna = Seq(sequence, generic_dna)
+            dna = Seq(sequence, IUPAC.ambiguous_dna)
             rna = dna.transcribe()
             yield SeqRecord(rna, id=name, description=description)
         elif transcribe == 'rna2dna':
-            rna = Seq(sequence, generic_rna)
+            rna = Seq(sequence, IUPAC.ambiguous_rna)
             dna = rna.back_transcribe()
             yield SeqRecord(dna, id=name, description=description)
 
+# Translate-related functions
+class CodonWarningTable(object):
+    """
+    Translation table for codons tht prints a warning when an unknown
+    codon is requested, then returns the value passed as missing_char
+    """
+
+    def __init__(self, wrapped, missing_char='X'):
+        self.wrapped = wrapped
+        self.missing_char = missing_char
+        self.seen = set()
+
+    def get(self, codon, missing=None):
+        try:
+            return self.__getitem__(codon)
+        except KeyError:
+            return missing
+
+    def __getitem__(self, codon):
+        if codon == '---':
+            return '-'
+        elif '-' in codon:
+            if codon not in self.seen:
+                logging.warn("Unknown Codon: %s", codon)
+                self.seen.add(codon)
+            return self.missing_char
+        else:
+            return self.wrapped.__getitem__(codon)
 
 def translate(records, translate):
     """
@@ -447,22 +615,26 @@ def translate(records, translate):
         rna2protein
         rna2proteinstop
     """
-    logging.info('Applying _translation generator: '
+    logging.info('Applying translation generator: '
                  'operation to perform is ' + translate + '.')
+
+    to_stop = translate.endswith('stop')
+
+    source_type = translate[:3]
+    alphabet = {'dna': IUPAC.ambiguous_dna, 'rna': IUPAC.ambiguous_rna}[source_type]
+
+    # Get a translation table
+    table = {'dna': CodonTable.ambiguous_dna_by_name["Standard"],
+             'rna': CodonTable.ambiguous_rna_by_name["Standard"]}[source_type]
+
+    # Handle ambiguities by replacing ambiguous codons with 'X'
+    table.forward_table = CodonWarningTable(table.forward_table)
+
     for record in records:
         sequence = str(record.seq)
-        description = record.description
-        name = record.id
-        if translate in ('dna2protein', 'dna2proteinstop'):
-            to_stop = translate == 'dna2proteinstop'
-            dna = Seq(sequence, generic_dna)
-            protein = dna.translate(to_stop=to_stop)
-            yield SeqRecord(protein, id=name, description=description)
-        elif translate in ('rna2protein', 'rna2proteinstop'):
-            to_stop = translate == 'rna2proteinstop'
-            rna = Seq(sequence, generic_rna)
-            protein = rna.translate(to_stop=to_stop)
-            yield SeqRecord(protein, id=name, description=description)
+        seq = Seq(sequence, alphabet)
+        protein = seq.translate(table, to_stop=to_stop)
+        yield SeqRecord(protein, id=record.id, description=record.description)
 
 
 def max_length_discard(records, max_length):
@@ -500,7 +672,7 @@ def min_ungap_length_discard(records, min_length):
     Discard any records that are shorter than min_length after removing gaps.
     """
     for record in records:
-        if len(record.seq.ungap('-')) >= min_length:
+        if len(ungap_all(record)) >= min_length:
             yield record
 
 
